'**************************************************************************
'
' A simple event processing demonstration.  Reads events from the queue and
' updates the various windows accordingly.
'
'****************************************************************************'

' GX IDs
CONST gxID             = &HCA               ' GX ID    
CONST gxVHID           = &HCA00             ' GX Virtual header ID value
CONST gxLIBID          = &HCA01             ' GX Library header ID value

' GX Display modes
CONST gxDETECT         = -1                 ' Detect mode when autoloading
CONST gxTEXT           = 0                  ' Text mode
CONST gxGRAPHICS       = 1                  ' Graphics mode

' GX Display types
CONST gxMINDISP        = 0
CONST gxCGA.4          = 0                  ' CGA Mode  4H ( 320x200x4  )
CONST gxCGA.6          = 1                  ' CGA Mode  5H ( 640x200x2  )
CONST gxEGA.D          = 2                  ' EGA Mode  DH ( 320x200x16 )
CONST gxEGA.E          = 3                  ' EGA Mode  EH ( 640x200x16 )
CONST gxEGA.F          = 4                  ' EGA Mode  FH ( 640x350x2  )
CONST gxEGA.10         = 5                  ' EGA Mode 10H ( 640x350x16 )
CONST gxVGA.11         = 6                  ' VGA Mode 11H ( 640x480x2  )
CONST gxVGA.12         = 7                  ' VGA Mode 12H ( 640x480x16 )
CONST gxVGA.13         = 8                  ' VGA Mode 13H ( 320x200x256)
CONST gxHERC           = 9                  ' Hercules     ( 720x348x2  )
CONST gxTS.29          = 10                 ' Tseng    29H ( 800x600x16 )
CONST gxTS.2D          = 11                 ' Tseng    2DH ( 640x350x256)
CONST gxTS.2E          = 12                 ' Tseng    2EH ( 640x480x256)
CONST gxTS.30          = 13                 ' Tseng    30H ( 800x600x256)
CONST gxTS.38          = 26                 ' Tseng    38H (1024x768x256)
CONST gxPAR.58         = 14                 ' Paradise 58H ( 800x600x16 )
CONST gxPAR.59         = 15                 ' Paradise 59H ( 800x600x2  )
CONST gxPAR.5E         = 16                 ' Paradise 5EH ( 640x400x256)
CONST gxPAR.5F         = 17                 ' Paradise 5FH ( 640x480x256)
CONST gxPAR.5C         = 27                 ' Paradise 5CH ( 800x600x256)
CONST gxPAR.60         = 28                 ' Paradise 60H (1024x768x256)
CONST gxV7.62          = 18                 ' Video 7  62H ( 800x600x16 )
CONST gxV7.66          = 19                 ' Video 7  66H ( 640x400x256)
CONST gxV7.67          = 20                 ' Video 7  67H ( 640x480x256)
CONST gxV7.69          = 21                 ' Video 7  69H ( 800x600x256)
CONST gxV7.6A          = 29                 ' Video 7  6AH (1024x768x256)
CONST gxATI.54         = 22                 ' ATI      54H ( 800x600x16 )
CONST gxATI.61         = 23                 ' ATI      61H ( 640x400x256)
CONST gxATI.62         = 24                 ' ATI      62H ( 640x480x256)
CONST gxATI.63         = 25                 ' ATI      63H ( 800x600x256)
CONST gxATI.64         = 30                 ' ATI      64H (1024x768x256)
CONST gxTRI.5B         = 31                 ' Trident  5BH ( 800x600x16 )
CONST gxTRI.5C         = 32                 ' Trident  5CH ( 640x400x256)
CONST gxTRI.5D         = 33                 ' Trident  5DH ( 640x480x256)
CONST gxTRI.5E         = 34                 ' Trident  5EH ( 800x600x256)
CONST gxTRI.62         = 35                 ' Trident  62H (1024x768x256)
CONST gxVESA.100       = 36                 ' VESA    100H ( 640x400x256)
CONST gxVESA.101       = 37                 ' VESA    101H ( 640x480x256)
CONST gxVESA.102       = 38                 ' VESA    102H ( 800x600x16 )
CONST gxVESA.103       = 39                 ' VESA    103H ( 800x600x256)
CONST gxVESA.105       = 40                 ' VESA    105H (1024x768x256)
CONST gxMAXDISP        = 40


' Searches
CONST gxFIRST          = 0                  ' Find first matching entry     
CONST gxNEXT           = 1                  ' Find next matching entry

' Logical Operations
CONST gxSET            = 0                  ' SET pixel value directly 
CONST gxAND            = 1                  ' AND pixel value with data
CONST gxOR             = 2                  ' OR  pixel value with data
CONST gxXOR            = 3                  ' XOR pixel value with data

' Virtual memory type
CONST gxCMM            = 0                  ' Conventional Memory virtual
CONST gxEMM            = 1                  ' Expanded Memory virtual
CONST gxDMM            = 2                  ' Disk paging virtual memory 
CONST gxXMM            = 3                  ' Extended Memory virtual

' Library type
CONST gxDIR            = 0                  ' DOS directory      
CONST gxGXL            = 1                  ' GXL file           
CONST gxEXE            = 2                  ' GXL imbedded in EXE

' Dither matrix type
CONST gxMAGICSQ        = 0                  ' Magic square
CONST gxDISPERSED      = 1                  ' Dispersed-dot
CONST gxCLUSTERED      = 2                  ' Clustered-dot

' Timer resolution
CONST gxLOWRES         = 0                  ' Low res (54.92 msec/cycle)
CONST gxHIGHRES        = 1                  ' High res (1 msec/cycle)

' Miscellaneous
CONST gxTRUE           = 1                  ' True value
CONST gxFALSE          = 0                  ' False value

' Video Info Constants
CONST viNONE           = 0                  ' No adapter/display
CONST viMDA            = 1                  ' Monochrome Display Adapter
CONST viCGA            = 2                  ' Color Graphics Adapter
CONST viEGA            = 3                  ' Enhanced Graphics Adapter
CONST viMCGA           = 4                  ' MCGA
CONST viVGA            = 5                  ' Video Graphics Array
CONST viHGC            = &H80               ' Hercules Monochrome
CONST viHGCPlus        = &H81               ' Hercules Plus
CONST viHGCColor       = &H82               ' Hercules InColor

CONST viMDAdisp        = 1                  ' Monochrome Display
CONST viCGAdisp        = 2                  ' CGA Display
CONST viEGAdisp        = 3                  ' EGA Display
CONST viVGAmono        = 4                  ' VGA Monochrome Display
CONST viVGAcolor       = 5                  ' VGA Color Display

' VGA Chip sets
CONST chipUNKNOWN      = 0                  ' Unknown chip set
CONST chipTSENG        = 1                  ' Tseng ET3000
CONST chipPARA         = 2                  ' Paradise
CONST chipV7           = 3                  ' Video 7
CONST chipATI          = 4                  ' ATI VGA Wonder
CONST chipTSENG4       = 5                  ' Tseng ET4000
CONST chipTRIDENT      = 6                  ' Trident
CONST chipVESA         = 7                  ' VESA compatible

' Hercules Configuration
CONST gxHERCdiag       = 0                  ' Hercules Diagnostic
CONST gxHERChalf       = 1                  ' Hercules Half
CONST gxHERCfull       = 3                  ' Hercules Full

' Default Palette Colors
CONST gxBLACK          = 0                  ' Black         = 00H
CONST gxBLUE           = 1                  ' Blue          = 01H
CONST gxGREEN          = 2                  ' Green         = 02H
CONST gxCYAN           = 3                  ' Cyan          = 03H
CONST gxRED            = 4                  ' Red           = 04H
CONST gxMAGENTA        = 5                  ' Magenta       = 05H
CONST gxBROWN          = 6                  ' Brown         = 06H
CONST gxGRAY           = 7                  ' Light Gray    = 07H
CONST gxDARKGRAY       = 8                  ' Dark Gray     = 038H
CONST gxLIGHTBLUE      = 9                  ' Light Blue    = 039H
CONST gxLIGHTGREEN     = 10                 ' Light Green   = 03AH
CONST gxLIGHTCYAN      = 11                 ' Light Cyan    = 03BH
CONST gxLIGHTRED       = 12                 ' Light Red     = 03CH
CONST gxLIGHTMAGENTA   = 13                 ' Light Magenta = 03DH
CONST gxYELLOW         = 14                 ' Yellow        = 03EH
CONST gxWHITE          = 15                 ' White         = 03FH

' Palette types
CONST gxPAL0           = 0                  ' No palette (B/W)
CONST gxPAL1           = 1                  ' CGA 4   color
CONST gxPAL2           = 2                  ' CGA 2   color
CONST gxPAL3           = 3                  ' EGA 16  color
CONST gxPAL4           = 4                  ' VGA 16  color
CONST gxPAL5           = 5                  ' VGA 256 color

' Error Codes
CONST gxSUCCESS        = 0                  ' Successful
CONST gxERR.OPEN       = -1                 ' Error opening file
CONST gxERR.BUFSMALL   = -2                 ' Buffer is too small
CONST gxERR.READ       = -3                 ' Error in reading from file
CONST gxERR.WRITE      = -4                 ' Error in writing to file
CONST gxERR.BADDISP    = -6                 ' Display not defined or invalid
CONST gxERR.BADMODE    = -7                 ' Bad display mode
CONST gxERR.BADPAGE    = -8                 ' Bad page
CONST gxERR.BADPAL     = -9                 ' Bad palette
CONST gxERR.NOPAL      = -10                ' No palette available
CONST gxERR.NOPALGRAB  = -11                ' Palette is not grabbed
CONST gxERR.NOTGXLIB   = -12                ' Not a GX library
CONST gxERR.NOTFOUND   = -14                ' Image not found in library
CONST gxERR.CLIPPARM   = -15                ' Bad Clipper parameter
CONST gxERR.ESCAPE     = -20                ' Escape key was pressed
CONST gxERR.NOEMM      = -21                ' No EMM driver was found
CONST gxERR.EMMFAIL    = -22                ' General EMM driver failure
CONST gxERR.EMMNOMEM   = -23                ' No EMM memory available
CONST gxERR.BADMEMTYPE = -24                ' Bad virtual memory type
CONST gxERR.CMMFAIL    = -25                ' General CMM alloc/free error
CONST gxERR.CMMNOMEM   = -26                ' No CMM memory available
CONST gxERR.BADCOORD   = -27                ' Bad coordinate location
CONST gxERR.NOTGX      = -28                ' Not a valid GX buffer/file
CONST gxERR.NODMM      = -29                ' No DMM driver was found
CONST gxERR.DMMFAIL    = -30                ' General DMM driver failure
CONST gxERR.DMMNOMEM   = -31                ' No DMM memory available
CONST gxERR.DMMFILE    = -32                ' Error with DMM file
CONST gxERR.CLOSE      = -33                ' Error closing a file
CONST gxERR.BADCHIP    = -34                ' Invalid chipset
CONST gxERR.EOF        = -35                ' End of file found
CONST gxERR.NOXMM      = -36                ' No XMS driver was found
CONST gxERR.XMMFAIL    = -37                ' General XMS driver failure
CONST gxERR.XMMNOMEM   = -38                ' No XMM memory available
CONST gxERR.NOVESA     = -39                ' No VESA driver was found
CONST gxERR.VESAFAIL   = -40                ' General VESA driver failure
CONST gxERR.BADDMTYPE  = -41                ' Invalid dither matrix type
CONST gxERR.NOTIMPL    = -900               ' Not implemented yet
CONST gxERR.GENERAL    = -999               ' General error

' GX Header
TYPE GXHEADER
  id           AS INTEGER                   ' Virtual header id (=CA00H)
  version      AS INTEGER                   ' Version number
  dtype        AS INTEGER                   ' Display type
  x1           AS INTEGER                   ' Picture dimensions (incl)
  y1           AS INTEGER
  x2           AS INTEGER
  y2           AS INTEGER
  hres         AS INTEGER                   ' Display horiz resolution
  vres         AS INTEGER                   ' Display vert  resolution
  nplanes      AS STRING*1                  ' Number of planes
  bitpx        AS STRING*1                  ' Bits per pixel
  bplin        AS INTEGER                   ' Bytes per line
  vtype        AS INTEGER                   ' Virtual buffer type
  vptr         AS LONG                      ' Virtual buffer pointer
  ptype        AS INTEGER                   ' Palette type
  pptr         AS LONG                      ' Palette buffer pointer
  reserved     AS STRING*94                 ' Reserved for GX kernel
END TYPE

' Library Header
TYPE GXLIBHEADER
  id           AS INTEGER                   ' Library header id (=CA01H)
  copyright    AS STRING*50                 ' Copyright notice
  version      AS INTEGER                   ' gxLib version
  label        AS STRING*40                 ' Library volume label
  entries      AS INTEGER                   ' Total image entries
  reserved     AS STRING*32                 ' Reserved for GX kernel
END TYPE

' Library Image Header
TYPE GXFINFO 
  pack         AS STRING*1                  ' Packing type
  filename     AS STRING*13                 ' Image file name
  filepos      AS LONG                      ' File position (image data)
  filesize     AS LONG                      ' File size
  filedate     AS INTEGER                   ' File date
  filetime     AS INTEGER                   ' File time
END TYPE

' Library Structure
TYPE GXLIB
  ltype        AS STRING*1                  ' Library type (gxDOS,GXL,LIB)
  path         AS STRING*65                 ' Directory path (gxDOS)      
  handle       AS INTEGER                   ' File handle (gxEXE,gxLIB)   
  entries      AS INTEGER                   ' Total index entries         
  libpos       AS LONG                      ' Library position            
  bbegent      AS INTEGER                   ' Bucket entry   (beginning)  
  bcurent      AS INTEGER                   ' Bucket entry   (current)    
  bmaxent      AS INTEGER                   ' Bucket entries (maximum)    
  bptr         AS LONG                      ' Bucket pointer              
  reserved     AS STRING*172                ' Reserved for GX kernel      
END TYPE

' Display Type Information
TYPE GXDINFO
  dtype        AS STRING*1                  ' Display type
  descrip      AS STRING*20                 ' String description
  ctype        AS STRING*1                  ' Color type
  mode         AS STRING*1                  ' Actual BIOS mode
  bitpx        AS STRING*1                  ' Number of bits per pixel
  hres         AS INTEGER                   ' Horizontal resolution
  vres         AS INTEGER                   ' Vertical   resolution
  bplin        AS INTEGER                   ' Number of bytes per row
  planes       AS STRING*1                  ' Number of display planes
  pages        AS STRING*1                  ' Number of display pages
  begseg       AS INTEGER                   ' Beginning display segment
  pagesize     AS INTEGER                   ' Size of display page
  paltype      AS INTEGER                   ' Palette format type
END TYPE

' Video Information
TYPE GXVINFO
  adapter      AS STRING*1                  ' Video Adapter
  display      AS STRING*1                  ' Video Display
END TYPE


' Chipset Info structure 
TYPE GXCINFO
  chipset      AS INTEGER                   ' Chipset            
  memory       AS INTEGER                   ' Memory installed   
  maxdisp      AS INTEGER                   ' Maxixum display type
END TYPE

' VESA SVGA information block
TYPE GXVESAINFO
  signature    AS STRING*4                  ' VESA signature string
  version      AS INTEGER                   ' Version number
  OEMstrptr    AS LONG                      ' Pointer to OEM string
  capabilities AS STRING*4                  ' Capabilities of video
  modeptr      AS LONG                      ' Ptr to supported SVGA modes
  memblocks    AS INTEGER                   ' Number of 64K memory blocks
  reserved     AS STRING*236                ' Reserved for VESA
END TYPE

' Byte/Char definition
TYPE BYTE
  char         AS STRING*1                  ' 1 Byte
END TYPE


' Function

' gxBT.asm
DECLARE FUNCTION gxBeginTimer%         (BYVAL resolution%)
DECLARE FUNCTION gxGetTime&
DECLARE FUNCTION gxElapsedTime&        (BYVAL marked&)
DECLARE FUNCTION gxDelay%              (BYVAL delay&)
DECLARE FUNCTION gxKeyCheck%           (BYVAL kfalg%)

' gxCV.asm
DECLARE FUNCTION gxSetVirtualHeader%   (BYVAL vtype%,SEG vhptr AS GXHEADER,BYVAL vptr&,BYVAL dtype%,BYVAL vwidth%,BYVAL vdepth%)
DECLARE FUNCTION gxCreateVirtual%      (BYVAL vtype%,SEG vhptr AS GXHEADER,BYVAL dtype%,BYVAL vwidth%,BYVAL vdepth%)
DECLARE FUNCTION gxDestroyVirtual%     (SEG vhptr AS GXHEADER)

' gxDD.asm
DECLARE FUNCTION gxDisplayDisplay%     (BYVAL x1%,BYVAL y1%,BYVAL x2%,BYVAL y2%,BYVAL spage%,BYVAL x%,BYVAL y%,BYVAL dpage%)

' gxDM.asm
DECLARE FUNCTION gxDMMInstalled%       
DECLARE FUNCTION gxInstallDMM%         (SEG path$,BYVAL frame%)
DECLARE FUNCTION gxRemoveDMM%

' gxDP.asm
DECLARE FUNCTION gxGetPaletteType%     (BYVAL ptype%)
DECLARE FUNCTION gxGetDisplayPalette%  (SEG pal%)
DECLARE FUNCTION gxSetDisplayPalette%  (SEG pal%)
DECLARE FUNCTION gxSetPaletteColor%    (BYVAL ndx%,BYVAL dcolor%)
DECLARE FUNCTION gxGetPaletteColor%    (BYVAL ndx%)
DECLARE FUNCTION gxSetPaletteRGB%      (BYVAL ndx%,BYVAL red%,BYVAL green%,BYVAL blue%)
DECLARE FUNCTION gxGetPaletteRGB%      (BYVAL ndx%,SEG red%,SEG green%,SEG blue%)

' gxDT.asm
DECLARE FUNCTION gxVerifyDisplayType%  (BYVAL dtype%)
DECLARE FUNCTION gxMapDisplayType%     (BYVAL nplanes%,BYVAL bitpx%)

' gxDV.asm
DECLARE FUNCTION gxDisplayVirtual%     (BYVAL x1%,BYVAL y1%,BYVAL x2%,BYVAL y2%,BYVAL page%,SEG vhptr AS GXHEADER,BYVAL x%,BYVAL y%)

' gxEM.asm
DECLARE FUNCTION gxEMSInstalled%

' gxGI.asm
DECLARE FUNCTION gxGetImage%           (SEG vhptr AS GXHEADER,BYVAL x1%,BYVAL y1%,BYVAL x2%,BYVAL y2%,BYVAL page%)

' gxLB.asm
DECLARE FUNCTION gxLibBuffer&          (SEG lib AS GXLIB,SEG filename$,BYVAL start&,BYVAL count%,SEG buffer%)

' gxOL.asm
DECLARE FUNCTION gxOpenLib%            (BYVAL ltype%,SEG libname$,SEG lib AS GXLIB,SEG ibuf%,BYVAL isize%)
DECLARE FUNCTION gxCloseLib%           (SEG lib AS GXLIB)
DECLARE FUNCTION gxFindFile%           (SEG lib AS GXLIB,SEG filemask$,SEG fi AS GXFINFO,BYVAL sflag%)

' gxPG.asm
DECLARE FUNCTION gxBeginPaletteGrab%   (SEG pal%)
DECLARE FUNCTION gxEndPaletteGrab%    

' gxPI.asm
DECLARE FUNCTION gxPutImage%           (SEG vhptr AS GXHEADER,BYVAL op%,BYVAL x%,BYVAL y%,BYVAL page%)

' gxQC.asm
DECLARE FUNCTION gxVESAInstalled%      (SEG viptr AS GXVESAINFO)
DECLARE FUNCTION gxQueryChipset%
DECLARE FUNCTION gxSetChipset%         (BYVAL chipset%)
DECLARE FUNCTION gxGetChipset%
DECLARE FUNCTION gxChipsetInfo%        (SEG ci AS GXCINFO)

' gxRD.asm
DECLARE FUNCTION gxClearDisplay%       (BYVAL dcolor%,BYVAL page%)

' gxRV.asm
DECLARE FUNCTION gxClearVirtual%       (SEG vhptr AS GXHEADER,BYVAL dcolor%)

' gxSB.asm
DECLARE FUNCTION gxSetBuffer%          (SEG buffer%,BYVAL bufmax%)

' gxSD.asm
DECLARE FUNCTION gxGetVersion%         (SEG major%,SEG minor%)
DECLARE FUNCTION gxSetDisplay%         (BYVAL dtype%)
DECLARE FUNCTION gxGetDisplay%
DECLARE FUNCTION gxGetDisplayInfo%     (BYVAL dtype%,SEG di AS GXDINFO)
DECLARE FUNCTION gxSetMode%            (BYVAL dmode%)
DECLARE FUNCTION gxGetMode%
DECLARE FUNCTION gxSetHerc%            (BYVAL hflag%)
DECLARE FUNCTION gxModeCheck%          (BYVAL modeflag%)
DECLARE FUNCTION gxRetraceCheck%       (BYVAL vsynflag%)

' gxSL.asm
DECLARE FUNCTION gxGetVirtualScanline% (SEG vhptr AS GXHEADER,BYVAL scanline%,SEG scanbuf%)
DECLARE FUNCTION gxSetVirtualScanline% (SEG vhptr AS GXHEADER,BYVAL scanline%,SEG scanbuf%)

' gxSP.asm
DECLARE FUNCTION gxSetPage%            (BYVAL page%)
DECLARE FUNCTION gxGetPage%           

' gxSR.asm
DECLARE FUNCTION gxSetRandom%          (BYVAL seed%)
DECLARE FUNCTION gxGetRandom%          (BYVAL range%)

' gxTB.asm
DECLARE FUNCTION gxTextBuffer%         (SEG textbuf%)
DECLARE FUNCTION gxBufferText%         (SEG textbuf%)

' gxVC.asm
DECLARE FUNCTION gxFindClosestColor%   (BYVAL scolor%)
DECLARE FUNCTION gxFindClosestRGB%     (BYVAL ptype%,BYVAL red%,BYVAL green%,BYVAL blue%)
DECLARE FUNCTION gxGetDitherMatrix%
DECLARE FUNCTION gxSetDitherMatrix%    (BYVAL dmtype%)
DECLARE FUNCTION gxGetConvertPalette%  (BYVAL ptype%,SEG palptr%)
DECLARE FUNCTION gxSetConvertPalette%  (BYVAL ptype%,SEG palptr%)
DECLARE FUNCTION gxVirtualConvert%     (SEG svhptr AS GXHEADER,SEG palptr,SEG dvhptr AS GXHEADER)

' gxVD.asm
DECLARE FUNCTION gxVirtualDisplay%     (SEG vhptr AS GXHEADER,BYVAL vx%,BYVAL vy%,BYVAL x1%,BYVAL y1%,BYVAL x2%,BYVAL y2%,BYVAL page%)

' gxVI.asm
DECLARE FUNCTION gxVideoInfo%          (SEG vi AS GXVINFO)

' gxVL.asm
DECLARE FUNCTION gxVirtualScale%       (SEG svhptr AS GXHEADER,SEG dvhptr AS GXHEADER)

' gxVR.asm
DECLARE FUNCTION gxVirtualSize&        (BYVAL dtype%,BYVAL vwidth%,BYVAL vdepth%)
DECLARE FUNCTION gxVirtualFree&        (BYVAL vtype%)

' gxVS.asm
DECLARE FUNCTION gxVirtualScroll%      (SEG vhptr AS GXHEADER,BYVAL vwidth%,BYVAL vdepth%,BYVAL x1%,BYVAL y1%,BYVAL x2%,BYVAL y2%,BYVAL vpage%)

' gxVV.asm
DECLARE FUNCTION gxVirtualVirtual%     (SEG svhptr AS GXHEADER,BYVAL x1%,BYVAL y1%,BYVAL x2%,BYVAL y2%,SEG dvhptr AS GXHEADER,BYVAL vx%,BYVAL vy%,BYVAL op%)

' gxXM.asm
DECLARE FUNCTION gxXMSInstalled%
DECLARE FUNCTION gxXMMInstalled%
DECLARE FUNCTION gxInstallXMM%         (BYVAL frame%)
DECLARE FUNCTION gxRemoveXMM%

' Supported GX display types 
CONST grMINDISP        = gxMINDISP          ' Don't use gxMAXDISP, since  
CONST grMAXDISP        = gxVESA.105         ' it may not supported        

' Line Styles
CONST grLSOLID         = &HFFFF             ' Solid        
CONST grLHUGEDASH      = &HFFF0             ' Huge   dashes
CONST grLBIGDASH       = &HFF00             ' Big    dashes
CONST grLMEDDASH       = &HF0F0             ' Medium dashes
CONST grLSMALLDASH     = &HCCCC             ' Small  dashes
CONST grLWIDEDOT       = &H8888             ' Wide  dots   
CONST grLCLOSEDOT      = &HAAAA             ' Close dots   
CONST grLDASHDOT       = &HFAFA             ' Dash dot     
CONST grLCENTER        = &HFDBF             ' Center line  

' Fill flags 
CONST grOUTLINE        = 1                  ' Outline      
CONST grFILL           = 2                  ' Fill interior

' Fill patterns 
CONST grFMIN           = 0                  ' Minimum fill 
CONST grFSOLID         = 0                  ' Solid
CONST grFLINE          = 1                  ' lines
CONST grFSLASH         = 2                  ' ///
CONST grFTHSLASH       = 3                  ' ///, thick
CONST grFBKSLASH       = 4                  ' \\\
CONST grFTHBKSLASH     = 5                  ' \\\, thick
CONST grFHATCH         = 6                  ' XXX
CONST grFTHHATCH       = 7                  ' XXX, thick
CONST grFINTER         = 8                  ' lines, interleaved
CONST grFWIDEDOT       = 9                  ' dots, wide
CONST grFCLOSEDOT      = 10                 ' dots, close
CONST grFUSER          = 11                 ' user-defined
CONST grFMAX           = 11                 ' Maximum fill

' Transparency types 
CONST grTRANS          = gxTRUE             ' Transparent
CONST grOPAQUE         = gxFALSE            ' Opaque

' Clipping flags 
CONST grCLIP           = gxTRUE             ' Enable  clipping
CONST grNOCLIP         = gxFALSE            ' Disable clipping

' Mouse cursors 
CONST grCMIN           = 0                  ' Minimum cursor
CONST grCARROW         = 0                  ' Left arrow
CONST grCCHECK         = 1                  ' Check mark
CONST grCPLUS          = 2                  ' Plus
CONST grCBOX           = 3                  ' Box
CONST grCRTHAND        = 4                  ' Right hand
CONST grCUPHAND        = 5                  ' Up hand
CONST grCSTOP          = 6                  ' Stop hand
CONST grCQUEST         = 7                  ' Question mark
CONST grCTIME          = 8                  ' Time
CONST grCUSER          = 9                  ' User cursor
CONST grCMAX           = 9                  ' Maximum cursor

' Mouse flags 
CONST grSHOW           = gxTRUE             ' Show cursor
CONST grHIDE           = gxFALSE            ' Hide cursor
CONST grTRACK          = gxTRUE             ' Track cursor
CONST grNOTRACK        = gxFALSE            ' No tracking
CONST grPROTECT        = gxTRUE             ' Protection enabled
CONST grNOPROTECT      = gxFALSE            ' Protection disabled
CONST grLBUTTON        = 1                  ' Left   button
CONST grMBUTTON        = 4                  ' Middle button
CONST grRBUTTON        = 2                  ' Right  button

' Colors 
CONST grBLACK          = gxBLACK            ' Black         = 00H
CONST grBLUE           = gxBLUE             ' Blue          = 01H
CONST grGREEN          = gxGREEN            ' Green         = 02H
CONST grCYAN           = gxCYAN             ' Cyan          = 03H
CONST grRED            = gxRED              ' Red           = 04H
CONST grMAGENTA        = gxMAGENTA          ' Magenta       = 05H
CONST grBROWN          = gxBROWN            ' Brown         = 06H
CONST grGRAY           = gxGRAY             ' Light Gray    = 07H
CONST grDARKGRAY       = gxDARKGRAY         ' Dark Gray     = 038H
CONST grLIGHTBLUE      = gxLIGHTBLUE        ' Light Blue    = 039H
CONST grLIGHTGREEN     = gxLIGHTGREEN       ' Light Green   = 03AH
CONST grLIGHTCYAN      = gxLIGHTCYAN        ' Light Cyan    = 03BH
CONST grLIGHTRED       = gxLIGHTRED         ' Light Red     = 03CH
CONST grLIGHTMAG       = gxLIGHTMAGENTA     ' Light Magenta = 03DH
CONST grYELLOW         = gxYELLOW           ' Yellow        = 03EH
CONST grWHITE          = gxWHITE            ' White         = 03FH                  

' Text Styles
CONST grTXT8X14        = 2                  ' 8X14 font
CONST grTXT8X8         = 3                  ' 8X8  font
CONST grTXT8X16        = 6                  ' 8X16 font

' Text justification
CONST grTLEFT          = 1                  ' Left  
CONST grTCENTER        = 2                  ' Center
CONST grTRIGHT         = 4                  ' Right 
CONST grTTOP           = 1                  ' Top   
CONST grTBOTTOM        = 4                  ' Bottom

'World coordinate system y axis orientation
CONST grSCREEN         = 0                  ' Y increases down
CONST grWORLD          = 1                  ' Y increases up

' Event queue constants
CONST grENOEVENTS      = &H0000             ' No events que'd           
CONST grEMOUSEMOVE     = &H0001             ' Mouse movement            
CONST grELPRESS        = &H0002             ' Left button presses       
CONST grELRELEASE      = &H0004             ' Left button releases      
CONST grERPRESS        = &H0008             ' Right button presses      
CONST grERRELEASE      = &H0010             ' Right button releases     
CONST grEKEYBOARD      = &H8000             ' Keystrokes generate events
CONST grEALLMOUSE      = &H001F             ' Movement, presses & releases
CONST grEALLEVENTS     = &H801F             ' All mouse and kbd events  

' Error Codes 
CONST grSUCCESS        = gxSUCCESS          ' Successful
CONST grERR.BADFONT    = -4000              ' Bad font type
CONST grERR.BADFILL    = -4001              ' Bad fill style
CONST grERR.NODRIVER   = -4002              ' No mouse driver found
CONST grERR.NOMOUSE    = -4003              ' No mouse found
CONST grERR.BADCURSOR  = -4004              ' Bad cursor style
CONST grERR.BADJUST    = -4005              ' Bad justification

CONST grERR.EQEMPTY    = -4020              ' Event queue empty
CONST grERR.EQFULL     = -4021              ' Event queue full
CONST grERR.EQBADINDEX = -4022              ' Out of range event queue index
CONST grERR.EQACTIVE   = -4023              ' Event Queue active

' State Structure 
TYPE GRSTATE
  vflag        AS INTEGER                   ' Virtual drawing flag
  vbtype       AS INTEGER                   ' Virtual bitmap type   
  vhptr        AS LONG                      ' Current virtual header
  page         AS INTEGER                   ' Page                  
  pageofs      AS INTEGER                   ' Page offset           
  fgcolor      AS INTEGER                   ' Foreground color      
  bkcolor      AS INTEGER                   ' Background color      
  firstpt      AS INTEGER                   ' First point plot flag 
  cpx          AS INTEGER                   ' Current position (x,y)
  cpy          AS INTEGER                   '                       
  op           AS INTEGER                   ' Logical operation     
  clip         AS INTEGER                   ' Clipping flag         
  clipx1       AS INTEGER                   ' Clipping region: (x1,y1)
  clipy1       AS INTEGER                   '                         
  clipx2       AS INTEGER                   '              to  (x2,y2)
  clipy2       AS INTEGER                   '                         
  fstyle       AS INTEGER                   ' Fill style              
  fcolor       AS INTEGER                   ' Fill color              
  ftrans       AS INTEGER                   ' Fill transparency       
  lstyle       AS INTEGER                   ' Line style              
  lthick       AS INTEGER                   ' Line thickness          
  viewrox      AS INTEGER                   ' Viewport origin: (ox,oy)
  viewroy      AS INTEGER                   '  (relative to viewport)                       
  viewpox      AS INTEGER                   ' Viewport origin: (ox,oy)
  viewpoy      AS INTEGER                   '  (physical screen coord)
  viewx1       AS INTEGER                   ' Viewport region: (x1,y1)
  viewy1       AS INTEGER                   '                         
  viewx2       AS INTEGER                   '              to  (x2,y2)
  viewy2       AS INTEGER                   '
  world        AS INTEGER                   ' World Coordinates
  worldx1      AS INTEGER                   '
  worldy1      AS INTEGER                   '
  worldx2      AS INTEGER                   '
  worldy2      AS INTEGER                   '
  worldsx      AS LONG                      '
  worldsy      AS LONG                      '
  worldyo      AS INTEGER                   ' World system y axis orientation
  tstyle       AS INTEGER                   ' Text font               
  ttrans       AS INTEGER                   ' Text transparency       
  thjust       AS INTEGER                   ' Text justification      
  tvjust       AS INTEGER                   ' 
END TYPE

' Event Structure 
TYPE GREVENT
  mask         AS INTEGER                   ' Event mask                  
  ascii        AS STRING*1                  ' ASCII code of keystroke     
  scan         AS STRING*1                  ' Scan code of keystroke      
  state        AS INTEGER                   ' KBD Shift/Mouse Button state
  curx         AS INTEGER                   ' X coord of Mouse cursor     
  cury         AS INTEGER                   ' Y coord of Mouse cursor     
  stime        AS LONG                      ' System timer tick timestamp 
END TYPE

' Functions
' grCA.asm
DECLARE FUNCTION grClearArea%          (BYVAL x1%, BYVAL y1%, BYVAL x2%, BYVAL y2%)

' grCL.asm
DECLARE FUNCTION grGetClipping%
DECLARE FUNCTION grSetClipping%        (BYVAL cflag%)
DECLARE FUNCTION grGetClipRegion%      (SEG x1%, SEG y1%, SEG x2%, SEG y2%)
DECLARE FUNCTION grSetClipRegion%      (BYVAL x1%, BYVAL y1%, BYVAL x2%, BYVAL y2%)
DECLARE FUNCTION grClipPoint%          (BYVAL x%, BYVAL y%)
DECLARE FUNCTION grClipLine%           (SEG x1%, SEG y1%, SEG x2%, SEG y2%)
DECLARE FUNCTION grClipRect%           (SEG x1%, SEG y1%, SEG x2%, SEG y2%)

' grDA.asm
DECLARE FUNCTION grDrawArc%            (BYVAL x1%, BYVAL y1%, BYVAL x2%, BYVAL y2%, BYVAL sangle%, BYVAL eangle%)
DECLARE FUNCTION grGetArcEnds%         (SEG x1%, SEG y1%, SEG x2%, SEG y2%)
DECLARE FUNCTION grDrawPieSlice%       (BYVAL x1%, BYVAL y1%, BYVAL x2%, BYVAL y2%, BYVAL sangle%, BYVAL eangle%, BYVAL fillflag%)

' grDE.asm
DECLARE FUNCTION grDrawCircle%         (BYVAL x%, BYVAL y%, BYVAL radius%, BYVAL fillflag%)
DECLARE FUNCTION grDrawEllipse%        (BYVAL x%, BYVAL y%, BYVAL a%, BYVAL b%, BYVAL fillflag%)

' grDL.asm
DECLARE FUNCTION grMoveTo%             (BYVAL x%, BYVAL y%)
DECLARE FUNCTION grMoveRel%            (BYVAL dx%, BYVAL dy%)
DECLARE FUNCTION grLineTo%             (BYVAL x%, BYVAL y%)
DECLARE FUNCTION grLineRel%            (BYVAL dx%, BYVAL dy%)
DECLARE FUNCTION grDrawLine%           (BYVAL x1%, BYVAL y1%, BYVAL x2%, BYVAL y2%)

'grDM.asm
DECLARE FUNCTION grInitMouse%
DECLARE FUNCTION grStopMouse%
DECLARE FUNCTION grTrackMouse%         (BYVAL tflag%)
DECLARE FUNCTION grDisplayMouse%       (BYVAL dflag%)
DECLARE FUNCTION grGetMouseMotion%     (SEG hmotion%, SEG vmotion%)
DECLARE FUNCTION grGetMouseButtons%
DECLARE FUNCTION grGetMousePresses%    (BYVAL button%, SEG presses%, SEG x%, SEG y%)
DECLARE FUNCTION grGetMouseReleases%   (BYVAL button%, SEG releases%, SEG x%, SEG y%)
DECLARE FUNCTION grGetMouseBounds%     (SEG x1%, SEG y1%, SEG x2%, SEG y2%)
DECLARE FUNCTION grSetMouseBounds%     (BYVAL x1%, BYVAL y1%, BYVAL x2%, BYVAL y2%)
DECLARE FUNCTION grGetMousePos%        (SEG x%, SEG y%)
DECLARE FUNCTION grSetMousePos%        (BYVAL x%, BYVAL y%)
DECLARE FUNCTION grGetMouseStyle%      (SEG style%, SEG mcolor%)
DECLARE FUNCTION grSetMouseStyle%      (BYVAL style%, BYVAL mcolor%)
DECLARE FUNCTION grGetMouseMask%       (BYVAL style%, SEG hotx%, SEG hoty%, SEG cursor%, SEG mask%)
DECLARE FUNCTION grSetMouseMask%       (BYVAL style%, BYVAL hotx%, BYVAL hoty%, SEG cursor%, SEG mask%)
DECLARE FUNCTION grSetMouseMode%       (BYVAL mode%)
DECLARE FUNCTION grProtectMouse%       (BYVAL pflag%)
DECLARE FUNCTION grSetMouseProtect%    (BYVAL x1%, BYVAL y1%, BYVAL x2%, BYVAL y2%)
DECLARE FUNCTION grGetMouseProtect%    (SEG x1%, SEG y1%, SEG x2%, SEG y2%)

' grDP.asm
DECLARE FUNCTION grDrawPoly%           (SEG points%, BYVAL numpts%, BYVAL fillflag%)

' grDR.asm
DECLARE FUNCTION grDrawSquare%         (BYVAL x%, BYVAL y%, BYVAL side%, BYVAL fillflag%)
DECLARE FUNCTION grDrawRect%           (BYVAL x1%, BYVAL y1%, BYVAL x2%, BYVAL y2%, BYVAL fillflag%)

' grEQ.asm
DECLARE FUNCTION grGetEventMask%
DECLARE FUNCTION grSetEventMask%      (BYVAL mask%)
DECLARE FUNCTION grWaitEvent%         (SEG grevent AS GREVENT)
DECLARE FUNCTION grGetEvent%          (SEG grevent AS GREVENT)
DECLARE FUNCTION grSetEvent%          (SEG grevent AS GREVENT)
DECLARE FUNCTION grPeekEvent%         (SEG grevent AS GREVENT, BYVAL index%)
DECLARE FUNCTION grSetEventBuffer%    (SEG buffer%, BYVAL length%)
DECLARE FUNCTION grGetEventStatus%    (SEG ecount%, SEG ofcount%)
DECLARE FUNCTION grClearEvents%

' grFF.asm
DECLARE FUNCTION grFloodFill%          (BYVAL x%, BYVAL y%, BYVAL border%)
DECLARE FUNCTION grFloodSpill%         (BYVAL x%, BYVAL y%)

' grGA.asm
DECLARE FUNCTION grGetActivePage%
DECLARE FUNCTION grSetActivePage%      (BYVAL apage%)
DECLARE FUNCTION grGetAspect%
DECLARE FUNCTION grSetAspect%          (BYVAL aspect%)
DECLARE FUNCTION grGetFillStyle%       (SEG style%, SEG fcolor%, SEG fillflag%)
DECLARE FUNCTION grSetFillStyle%       (BYVAL style%, BYVAL fcolor%, BYVAL ftype%)
DECLARE FUNCTION grGetFillPattern%     (BYVAL style%, SEG pattern%)
DECLARE FUNCTION grSetFillPattern%     (BYVAL style%, SEG pattern%)
DECLARE FUNCTION grGetLineStyle%       (SEG style%, SEG thickness%)
DECLARE FUNCTION grSetLineStyle%       (BYVAL style%, BYVAL thickness%)
DECLARE FUNCTION grGetOp%
DECLARE FUNCTION grSetOp%              (BYVAL op%)
DECLARE FUNCTION grGetBkColor%
DECLARE FUNCTION grSetBkColor%         (BYVAL bcolor%)
DECLARE FUNCTION grGetColor%
DECLARE FUNCTION grSetColor%           (BYVAL fcolor%)
DECLARE FUNCTION grGetState%           (SEG state AS GRSTATE)
DECLARE FUNCTION grSetState%           (SEG state AS GRSTATE)
DECLARE FUNCTION grDefaultState%
DECLARE FUNCTION grGetCP%              (SEG x%, SEG y%)
DECLARE FUNCTION grMaxX%
DECLARE FUNCTION grMaxY%
DECLARE FUNCTION grMaxColor%

' grGP.asm
DECLARE FUNCTION grGetPixel%           (BYVAL x%, BYVAL y%)

' grOT.asm
DECLARE FUNCTION grGetTextStyle%       (SEG style%, SEG trans%)
DECLARE FUNCTION grSetTextStyle%       (BYVAL style%, BYVAL trans%)
DECLARE FUNCTION grGetTextJustify%     (SEG thjust%, SEG tvjust%)
DECLARE FUNCTION grSetTextJustify%     (BYVAL thjust%, BYVAL tvjust%)
DECLARE FUNCTION grOutText%            (SEG text$)

' grPP.asm
DECLARE FUNCTION grPutPixel%           (BYVAL x%, BYVAL y%, BYVAL pcolor%)

' grSV.asm
DECLARE FUNCTION grGetVersion%         (SEG major%,SEG minor%)
DECLARE FUNCTION grGetActiveVirtual%   (SEG vhptr AS GXHEADER)
DECLARE FUNCTION grSetActiveVirtual%   (SEG vhptr AS GXHEADER)
DECLARE FUNCTION grGetVirtual%
DECLARE FUNCTION grSetVirtual%         (BYVAL vflag%)

' grVP.asm
DECLARE FUNCTION grGetViewPort%        (SEG x1%, SEG y1%, SEG x2%, SEG y2%)
DECLARE FUNCTION grSetViewPort%        (BYVAL x1%, BYVAL y1%, BYVAL x2%, BYVAL y2%)
DECLARE FUNCTION grClearViewPort%
DECLARE FUNCTION grGetWorld%
DECLARE FUNCTION grSetWorld%           (BYVAL wflag%)
DECLARE FUNCTION grGetViewWorld%       (SEG x1%, SEG y1%, SEG x2%, SEG y2%)
DECLARE FUNCTION grSetViewWorld%       (BYVAL x1%, BYVAL y1%, BYVAL x2%, BYVAL y2%)
DECLARE FUNCTION grGetViewOrigin%      (SEG x%, SEG y%)
DECLARE FUNCTION grSetViewOrigin%      (BYVAL x%, BYVAL y%)
DECLARE FUNCTION grWorldLocalX%        (BYVAL wx%)
DECLARE FUNCTION grWorldLocalY%        (BYVAL wy%)
DECLARE FUNCTION grLocalWorldX%        (BYVAL lx%)
DECLARE FUNCTION grLocalWorldY%        (BYVAL ly%)
DECLARE FUNCTION grGetWorldOrientation%
DECLARE FUNCTION grSetWorldOrientation%(BYVAL orient%)


  
CONST INITEMASK = (grELPRESS OR grERPRESS OR grEKEYBOARD) ' Initial event mask
  
CONST VMARGIN% = 3                           ' Text margin in y direction   
CONST HMARGIN% = 5                           ' Text margin in x direction   
CONST TWIDTH%  = 8                           ' Text width                   
CONST THEIGHT% = 8                           ' Text height                  
  
CONST TITLEHEIGHT% = (THEIGHT+2*VMARGIN)     ' Height of window titles      
  
CONST  MWW%    = 640                         ' Width of main window         
  
CONST  EMWX%   = 37                          ' Event mask window location   
CONST  EMWY%   = 18                          ' relative to main winwdow     
CONST  EMWW%   = 176                         ' Event mask window width      
CONST  EMWH%   = 175                         ' Event mask window height     
  
CONST  QSWX%   = 37                          ' Queue Status window location 
CONST  QSWY%   = 199                         ' relative to main window      
CONST  QSWW%   = 176                         ' Queue status window width    
CONST  QSWH%   = 143                         ' Queue status window height   
  
CONST  SKWX%   = 240                         ' Sketch window location       
CONST  SKWY%   = 18                          ' relative to main window      
CONST  SKWW%   = 375                         ' Sketch window width          
CONST  SKWH%   = 256                         ' Sketch window height         
  
CONST  KBWX%   = 240                         ' Keyboard window location     
CONST  KBWY%   = 280                         ' relative to main window      
CONST  KBWW%   = 375                         ' Keyboard window width        
CONST  KBWH%   = 36                          ' Keyboard window height       
  
CONST  MAXBUTTONS%  = 11                     ' Total number of buttons      
CONST  BWIDTH%      = 150                    ' Button width                 
CONST  BDEPTH%      = 18                     ' Button depth                 
CONST  BBEVEL%      = 3                      ' Button bevel                 
CONST  BDOWN%       = gxTRUE                 ' Button down flag             
CONST  BUP%         = gxFALSE                ' Button up   flag             
CONST  BDELAY%      = 250                    ' Button delay                 
  
CONST  BROWSPACE%   = (BDEPTH+1)             ' Button row spacing           
CONST  BEDGESPACE%  = 20                     ' Button edge spacing          
  
CONST  BNOEVENTS%   = 0                      ' No Events Button             
CONST  BMOUSEMOVE%  = 1                      ' Mouse Movement Button        
CONST  BLPRESS%     = 2                      ' Left Presses Button          
CONST  BLRELEASE%   = 3                      ' Left Releases Button         
CONST  BRPRESS%     = 4                      ' Right Presses Button         
CONST  BRRELEASE%   = 5                      ' Right Releases Button        
CONST  BKBDEVENTS%  = 6                      ' Keyboard Events Button       
CONST  BALLEVENTS%  = 7                      ' All Events Button            
CONST  BEVENTPROC%  = 8                      ' Event Processing Button      
CONST  BRESET%      = 9                      ' Reset Button                 
CONST  BQUIT%       = 10                     ' Quit Button                  
  
CONST  BEPX%   = 10                          ' Event proc. button position  
CONST  BEPY%   = 116                         ' relative to queue stat win.  
  
CONST  BNEX%   = 13                          ' No events button position    
CONST  BNEY%   = 20                          ' relative to mask window      
  
CONST  BRX%    = (KBWX+42)                   ' Reset button pos relative    
CONST  BRY%    = 324                         ' to main window               
  
CONST  BQX%    = (BRX+BWIDTH+HMARGIN)        ' Quit button pos relative to  
CONST  BQY%    = BRY                         ' main window                  
  
CONST  ASCII.ESC%   = &H1B                   ' Exit key                     
  
' Structures  
TYPE PIXEL
  x AS INTEGER                              ' X coordinate                 
  y AS INTEGER                              ' Y coordinate                 
END TYPE
  
TYPE RECT
  x1 AS INTEGER                             ' Upper left  X coordinate     
  y1 AS INTEGER                             ' Upper left  Y coordinate     
  x2 AS INTEGER                             ' Lower right X coordinate     
  y2 AS INTEGER                             ' Lower right Y coordinate     
END TYPE
  
DECLARE FUNCTION PinR%                      (x%, y%, r AS RECT)
  
DIM SHARED grtype%                          ' GX Kernel display type       
DIM SHARED maxx%, maxy%                     ' Maximum x and y coords       
DIM SHARED maxcolor%                        ' Maximum pixel value          
DIM SHARED retcode%                         ' Return code                  
DIM SHARED done%                            ' Execute until done           
DIM SHARED deltaof%, ofcount%               ' Overflow delta and count     
DIM SHARED ecount%                          ' Events in queue count        
DIM SHARED emask%                           ' Event mask                   
DIM SHARED ep%                              ' Event processing status      
DIM SHARED skcolor%                         ' Draw color in sketch window  
DIM SHARED mwbkc%                           ' Main window bk color         
DIM SHARED emwbkc%                          ' Event mask window bk color   
DIM SHARED qswbkc%                          ' Queue status window bk color 
DIM SHARED skwbkc%                          ' Sketch window bk color       
DIM SHARED kbwbkc%                          ' Keyboard window bk color     
DIM SHARED buttonc%                         ' Button Color                 
DIM SHARED bevelc%                          ' Bevel color                  
DIM SHARED oldcount%                        ' The last event count printed 
  
DIM SHARED mwx%                             ' Main window location         
DIM SHARED mwy%                             ' relative to screen           
  
DIM SHARED bname$(MAXBUTTONS)               ' Button name array            
  bname$(BNOEVENTS ) = "No Events"
  bname$(BMOUSEMOVE) = "Mouse Movement"
  bname$(BLPRESS   ) = "Left Presses"
  bname$(BLRELEASE ) = "Left Releases"
  bname$(BRPRESS   ) = "Right Presses"
  bname$(BRRELEASE ) = "Right Releases"
  bname$(BKBDEVENTS) = "Keyboard Events"
  bname$(BALLEVENTS) = "All Events"
  bname$(BEVENTPROC) = "Event Processing"
  bname$(BRESET    ) = "Reset"
  bname$(BQUIT     ) = "Quit"
  
DIM SHARED s$                               ' General purpose string buff  
  
DIM SHARED tcp              AS PIXEL        ' Text current position        
DIM SHARED scp              AS PIXEL        ' Sketch current position      
DIM SHARED bpos(MAXBUTTONS) AS PIXEL        ' Button positions             
  
DIM SHARED sketchR          AS RECT         ' Valid sketch region          
DIM SHARED epR              AS RECT         ' Event Proc. Button region    
DIM SHARED resetR           AS RECT         ' Reset button region          
DIM SHARED quitR            AS RECT         ' Quit button region           
DIM SHARED maskR            AS RECT         ' Event mask Buttons region    
  
DIM SHARED grevent          AS GREVENT      ' Keyboard or mouse event      
  
'******** 
  
'
' This routine initializes the button positions.
'
'
'
  
SUB InitButtons
  
  DIM  i%,brow%,bcol%
  
  ' Set first button position  
  brow% = BNEY
  bcol% = BNEX
  
  ' Set button positions  
  FOR i% = BNOEVENTS TO BALLEVENTS
  
    ' Set button position  
    bpos(i%).x% = bcol%
    bpos(i%).y% = brow%
    brow% = brow% + BROWSPACE
  
  NEXT i%
  
  ' Place Event Processing Button  
  bpos(BEVENTPROC).x% = BEPX
  bpos(BEVENTPROC).y% = BEPY
  
  ' Place Reset Button  
  bpos(BRESET).x% = BRX
  bpos(BRESET).y% = BRY
  
  ' Place Quit Button  
  bpos(BQUIT).x% = BQX
  bpos(BQUIT).y% = BQY
  
END SUB ' end of InitButtons  
  
'******** 
  
'
' This routine initializes the target rectangles.
'
'
'
  
SUB InitRects
  
  ' Sketch rectangle  
  sketchR.x1% = SKWX+mwx% + 1
  sketchR.y1% = SKWY+mwx% + TITLEHEIGHT
  sketchR.x2% = SKWX+mwx% + SKWW - 2
  sketchR.y2% = SKWY+mwy% + SKWH - 2 - TITLEHEIGHT
  
  ' Event processing button rectangle  
  epR.x1% = QSWX+mwx% + BEPX
  epR.y1% = QSWY+mwy% + BEPY
  epR.x2% = epR.x1% + BWIDTH - 1
  epR.y2% = epR.y1% + BDEPTH - 1
  
  ' Mask buttons rectangle  
  maskR.x1% = EMWX + mwx% + BNEX
  maskR.y1% = EMWY + mwy% + BNEY
  maskR.x2% = maskR.x1% + BWIDTH - 1
  maskR.y2% = maskR.y1% + 7 * BROWSPACE + BDEPTH - 1
  
  ' Reset button rectangle  
  resetR.x1% = BRX + mwx%
  resetR.y1% = BRY + mwy%
  resetR.x2% = resetR.x1% + BWIDTH - 1
  resetR.y2% = resetR.y1% + BDEPTH - 1
  
  ' Quit button rectangle  
  quitR.x1% = BQX + mwx%
  quitR.y1% = BQY + mwy%
  quitR.x2% = quitR.x1% + BWIDTH - 1
  quitR.y2% = quitR.y1% + BDEPTH - 1
  
END SUB ' end of InitRects  
  
'******** 
  
'
' This routine determines if a point lies within a rectangle
'
'
'
  
FUNCTION PinR%(x%, y%, r AS RECT)
  
  DIM  ret%
  
  ret% = gxTRUE
  
  IF ((x% < r.x1%) OR (x% > r.x2%) OR (y% < r.y1%) OR (y% > r.y2%)) THEN
    ret% = gxFALSE
  ENDIF
  
  PinR% = ret%
  
END FUNCTION ' end of PinR  
  
'******** 
  
'
' Draw a window with a title.
'
'
  
SUB DrawWindow(x1%, y1%, x2%, y2%, title$, fg%, bg%)
  
  retcode% = grSetColor%(fg%)
  retcode% = grSetBkColor%(bg%)
  retcode% = grSetFillStyle%(grFSOLID, bg%, grOPAQUE)
  
  retcode% = grDrawRect%(x1%, y1%, x2%, y2%, grFILL+grOUTLINE)
  retcode% = grDrawLine%(x1%+1, y1%+(2*VMARGIN)+THEIGHT-1, x2%-1, y1%+(2*VMARGIN)+THEIGHT-1)
  retcode% = grMoveTo%(x1%+((x2%-x1%+1)\2), y1%+VMARGIN)
  retcode% = grOutText%(title$)
  
END SUB ' end of DrawWindow  
  
'******** 
  
'
' Draw a text string at specified row and column
'
'
  
SUB DrawString( r%, c%, s$, fg%, bg%)
  
  retcode% = grSetColor%(fg%)
  retcode% = grSetBkColor%(bg%)
  retcode% = grSetTextJustify%(grTLEFT, grTTOP)
  
  retcode% = grMoveTo%(HMARGIN+c%*TWIDTH, 2+VMARGIN+r%*(VMARGIN+THEIGHT))
  retcode% = grOutText%(s$)
  
END SUB ' end of DrawString  
  
'******** 
  
'
'
' This routine draws a 3D box with top-left corner (x1,y1) and
' bottom-right corner (x2,y2).  A border width of bw and a border
' style of bs (either concave or convex) is used.  The sunc color
' is sunc, the shadow color is shadowc, and the interior color is fillc.
'
'
  
SUB DrawBevel(x1%, y1%, x2%, y2%, bw%, sunc%, shadowc%, fillc%, pressed%)
  
  DIM  scolor%
  DIM  fstyle%,fcolor%,ftrans%
  DIM  i%,t%
  DIM  sun%,shadow%
  
  ' Save current settings  
  scolor% = grGetColor%
  retcode% = grGetFillStyle%(fstyle%,fcolor%,ftrans%)
  
  shadow% = shadowc%
  sun% = sunc%

  ' If button is pressed, then switch colors  
  IF (pressed% <> 0) THEN
    t% = shadow%
    shadow% = sun%
    sun% = t%
  END IF
  
  ' Draw bevelled surfaces  
  FOR i% = 0 TO bw% - 1
  
    ' Draw top/left sides  
    retcode% = grSetColor%(sun%)
    retcode% = grMoveTo%(x1%+i%,y2%-i%)
    retcode% = grLineTo%(x1%+i%,y1%+i%)
    retcode% = grLineTo%(x2%-i%,y1%+i%)
  
    ' Draw bottom right sides  
    retcode% = grSetColor%(shadow%)
    retcode% = grMoveTo%(x2%-i%,y1%+i%+1)
    retcode% = grLineTo%(x2%-i%,y2%-i%)
    retcode% = grLineTo%(x1%+i%+1,y2%-i%)
  
  NEXT i%
  
  ' Draw button  
  retcode% = grSetFillStyle%(grFSOLID,fillc%,grOPAQUE)
  retcode% = grDrawRect%(x1%+bw%,y1%+bw%,x2%-bw%,y2%-bw%,grFILL)
  
  ' Restore settings  
  retcode% = grSetFillStyle%(fstyle%,fcolor%,ftrans%)
  retcode% = grSetColor%(scolor%)
  
END SUB ' end of DrawBevel  
  
'******** 
  
'
' This routine Displays the given button, either up or down.
'
'
'
  
SUB DrawButton(b%, ud%)
  
  DIM  fcolor%,bcolor%
  DIM  hjust%,vjust%
  
  retcode% = grDisplayMouse%(grHIDE)
  
  ' Save current settings  
  fcolor% = grGetColor%
  bcolor% = grGetBkColor%
  retcode% = grGetTextJustify%(hjust%, vjust%)
  
  ' Set up text for buttons  
  IF (maxcolor% = 1) THEN
    retcode% = grSetColor%(grBLACK)
  ELSE
    IF (ud% <> 0) THEN
      retcode% = grSetColor%(grBLACK)
    ELSE
      retcode% = grSetColor%(grWHITE)
    END IF
  END IF
  
  retcode% = grSetBkColor%(buttonc%)
  retcode% = grSetTextStyle%(grTXT8X8,grOPAQUE)
  retcode% = grSetTextJustify%(grTCENTER,grTCENTER)
  
  ' Draw button  
  CALL DrawBevel(bpos(b%).x%,bpos(b%).y%,bpos(b%).x%+BWIDTH-1,bpos(b%).y%+BDEPTH-1,BBEVEL,maxcolor%,bevelc%,buttonc%,ud%)
  retcode% = grMoveTo%(bpos(b%).x%+BWIDTH\2,bpos(b%).y%+BDEPTH\2)
  retcode% = grOutText%(bname$(b%))
  
  ' Restore settings  
  retcode% = grSetTextJustify%(hjust%,vjust%)
  retcode% = grSetColor%(fcolor%)
  retcode% = grSetBkColor%(bcolor%)
  
  retcode% = grDisplayMouse%(grSHOW)
  
END SUB ' end of DrawButton  
  
'******** 
  
'
' This routine draws the event buttons in their appropriate positions.
'
'
'
  
SUB DisplayMaskButtons
  
  DIM i%
  
  retcode% = grSetViewPort%(EMWX+mwx%, EMWY+mwy%, maxx%, maxy%)
  
  FOR i% = BMOUSEMOVE TO BKBDEVENTS-1
    CALL DrawButton(i%, emask% AND (1*(2^(i%-1))))
  NEXT i%
  
  CALL DrawButton(BKBDEVENTS, emask% AND grEKEYBOARD)
  
END SUB ' end of DisplayMaskButtons  
  
'******** 
  
'
' This routine Handles clicking of the Mask buttons.
'
'
'
  
SUB HandleMaskButtons(x%, y%)
  
  
  DIM b%                                    ' Button index                 
  DIM bit%                                  ' Bit position in event mask   
  
  b% = (y% - maskR.y1%) \ BROWSPACE
  
  retcode% = grSetViewPort%(EMWX+mwx%, EMWY+mwy%, maxx%, maxy%)
  
  SELECT CASE b%
    CASE BNOEVENTS
      ' Mask out all events  
      CALL DrawButton(BNOEVENTS, BDOWN)
      emask% = grENOEVENTS
      CALL DisplayMaskButtons
      WHILE ((grGetMouseButtons% AND grLBUTTON) <> 0)
      WEND
      CALL DrawButton(BNOEVENTS, BUP)
  
    CASE BALLEVENTS
      ' Enable all events  
      CALL DrawButton(BALLEVENTS, BDOWN)
      emask% = grEALLEVENTS
      CALL DisplayMaskButtons
      WHILE ((grGetMouseButtons% AND grLBUTTON) <> 0)
      WEND
      CALL DrawButton(BALLEVENTS, BUP)
  
    CASE BKBDEVENTS
      ' Toggle keyboard events  
      emask% = emask% XOR grEKEYBOARD
      CALL DrawButton(BKBDEVENTS, emask% AND grEKEYBOARD)
  
    CASE ELSE
      ' Toggle individual mouse events  
      bit% = 1 * ( 2^(b%-1))
      emask% = emask% XOR bit%
      CALL DrawButton(b%, emask% AND bit%)
  
  END SELECT
  
  retcode% = grSetEventMask%(emask%)
  
END SUB ' end of HandleMaskButtons  
  
'******** 
  
'
' This routine Handles selection of the quit button.
'
'
'
  
SUB HandleQuitButton
  
  retcode% = grSetViewPort%(mwx%, mwy%, maxx%, maxy%)
  CALL DrawButton(BQUIT, BDOWN)
  done% = gxTRUE
  WHILE ((grGetMouseButtons% AND grLBUTTON) <> 0)
  WEND
  CALL DrawButton(BQUIT, BUP)
  
END SUB ' end of HandleQuitButton  
  
'******** 
  
'
' This routine resets the keyboard window
'
'
'
  
SUB ResetKBW
  
  ' Reset current position  
  tcp.x% = HMARGIN
  tcp.y% = TITLEHEIGHT + 2*VMARGIN
  
  retcode% = grSetViewPort%(KBWX+mwx%, KBWY+mwy%, maxx%, maxy%)
  retcode% = grSetFillStyle%(grFSOLID, kbwbkc%, grOPAQUE)
  retcode% = grDisplayMouse%(grHIDE)
  
  ' Erase the text field  
  retcode% = grDrawRect%(tcp.x%, tcp.y%, KBWW-2, tcp.y%+THEIGHT-1, grFILL)
  
  retcode% = grDisplayMouse%(grSHOW)
  retcode% = grMoveTo%(tcp.x%, tcp.y%)
  
END SUB ' end of ResetKBW  
  
'******** 
  
'
' This routine resets the Sketch window
'
'
'
  
SUB ResetSKW
  
  retcode% = grSetViewPort%(0, 0, maxx%, maxy%)
  retcode% = grSetFillStyle%(grFSOLID, skwbkc%, grOPAQUE)
  retcode% = grDisplayMouse%(grHIDE)
  ' Erase sketch window  
  retcode% = grDrawRect%(sketchR.x1%, sketchR.y1%, sketchR.x2%, sketchR.y2%, grFILL)
  retcode% = grDisplayMouse%(grSHOW)
  
  ' Reset Drawing position to center of sketch window  
  scp.x% = SKWX+mwx%+(SKWW\2)
  scp.y% = SKWY+mwy%+(SKWH\2)
  
  
END SUB ' end of ResetSKW  
  
'******** 
  
'
' Update the event and overflow counts.
'
'
'
  
SUB UpdateQueueStatus
  
  retcode% = grSetViewPort%(QSWX+mwx%, QSWY+mwy%, maxx%, maxy%)
  
  
  ' Update event count if it has changed  
  IF (oldcount% <> ecount%) THEN
    s$ = RIGHT$(" " + STR$(ecount%), 2)
    retcode% = grDisplayMouse%(grHIDE)
    CALL DrawString( 8, 10, s$, maxcolor%, qswbkc%)
    retcode% = grDisplayMouse%(grSHOW)
    oldcount% = ecount%
  END IF
  
  ' Update overflow count if it has changed  
  IF (deltaof% <> 0) THEN
    s$ = RIGHT$("   " + STR$(ofcount%), 4)
    retcode% = grDisplayMouse%(grHIDE)
    CALL DrawString( 9, 10, s$, maxcolor%, qswbkc%)
    retcode% = grDisplayMouse%(grSHOW)
  END IF
  
END SUB ' end of UpdateQueueStatus  
  
'******** 
  
'
' This routine Handles selection of the reset button.
'
'
'
  
SUB HandleResetButton
  
  retcode% = grSetViewPort%(mwx%, mwy%, maxx%, maxy%)
  CALL DrawButton(BRESET, BDOWN)
  
  ' Reset Keyboard and sketch windows  
  CALL ResetKBW
  CALL ResetSKW
  
  WHILE ((grGetMouseButtons% AND grLBUTTON) <> 0)
  WEND
  retcode% = grSetViewPort%(mwx%, mwy%, maxx%, maxy%)
  CALL DrawButton(BRESET, BUP)
  
  ' Clear the queue and reset/update counters  
  emask% = INITEMASK
  retcode% = grSetEventMask%(emask%)
  CALL DisplayMaskButtons
  retcode% = grClearEvents%
  deltaof% = -1
  ofcount% = 0
  CALL UpdateQueueStatus
  
END SUB ' end of HandleResetButton  
  
'******** 
  
'
' This routine Handles selection of the Event Processing button.
'
'
'
  
SUB HandleEPButton
  
  ' Toggle event processing  
  ep% = ep% XOR 1
  CALL DrawButton(BEVENTPROC, ep%)
  
END SUB ' end of HandleEPButton  
  
'******** 
  
'
' Display the event fields.
'
'
'
  
SUB DisplayEvent(grevent AS GREVENT)
  
  retcode% = grSetViewPort%(QSWX+mwx%, QSWY+mwy%, maxx%, maxy%)
  
  s$ = RIGHT$("000" + HEX$(grevent.mask%), 4)
  
  retcode% = grDisplayMouse%(grHIDE)
  
  CALL DrawString( 2, 7, s$, maxcolor%, qswbkc%)
  
  IF (ASC(grevent.ascii$) <> 0) THEN
    s$ = "'" + grevent.ascii$ + ""
  ' If extended key display SPACE  
  ELSE
    s$ = "' "
  END IF
  
  CALL DrawString( 3, 7, s$, maxcolor%, qswbkc%)
  
  s$ = RIGHT$("0" + HEX$(ASC(grevent.scan$)), 2)
  CALL DrawString( 3, 17, s$, maxcolor%, qswbkc%)
  
  s$ = RIGHT$("000" + HEX$(grevent.state%), 4)
  CALL DrawString( 4, 7, s$, maxcolor%, qswbkc%)
  
  s$ = RIGHT$("00" + STR$(grevent.curx%), 3)
  CALL DrawString( 5, 7, s$, maxcolor%, qswbkc%)
  
  s$ = RIGHT$("00" + STR$(grevent.cury%), 3)
  CALL DrawString( 5, 17, s$, maxcolor%, qswbkc%)
  
  s$ = RIGHT$("0000000" + HEX$(grevent.stime&), 8)
  CALL DrawString( 6, 7, s$, maxcolor%, qswbkc%)
  
  retcode% = grDisplayMouse%(grSHOW)
  
END SUB ' end of DisplayEvent  
  
'******** 
  
'
' Set the drawing color for the sketch window - in 16 color modes
' cycle through colors 1 to maxcolor()
'
'
  
SUB SetSKColor
  
  IF (maxcolor% > 1) THEN
    IF (skcolor% >= maxcolor%) THEN
      skcolor% = 1
    ELSE
      skcolor% = skcolor% + 1
    END IF
  END IF
  
  retcode% = grSetColor%(skcolor%)
  
END SUB ' end of SetSKColor  
  
'******** 
  
'
' Process the current event.
'
'
'
  
SUB HandleEvent(grevent AS GREVENT)
  
  ' Are keyboard events enabled?  
  IF ((grevent.mask% AND grEKEYBOARD) <> 0) THEN
  
    ' Was it the EXIT key?  
    IF (ASC(grevent.ascii$) = ASCII.ESC) THEN
      done% = gxTRUE
  
    ' Not the EXIT key so add to the keyboard window  
    ELSE
      retcode% = grSetViewPort%(KBWX+mwx%, KBWY+mwy%, maxx%, maxy%)
      retcode% = grMoveTo%(tcp.x%, tcp.y%)
      s$ = grevent.ascii$
      ' Are we at the right edge of the keyboard window?  
      IF (tcp.x% >= KBWW-HMARGIN-TWIDTH) THEN
        CALL ResetKBW
      END IF
      tcp.x% = tcp.x% + TWIDTH
      retcode% = grSetBkColor%(kbwbkc%)
      retcode% = grDisplayMouse%(grHIDE)
      retcode% = grOutText%(s$)
      retcode% = grDisplayMouse%(grSHOW)
    END IF
  
  ' Did the event occur in the sketch window?  
  ELSEIF (PinR%(grevent.curx%, grevent.cury%, sketchR) <> 0) THEN
  
    retcode% = grSetViewPort%(0, 0, maxx%, maxy%)
    CALL SetSKColor
  
  
    ' Left press - update CP and draw pixel  
    IF ((grevent.mask% AND grELPRESS) <> 0) THEN
      scp.x% = grevent.curx%
      scp.y% = grevent.cury%
  
      retcode% = grDisplayMouse%(grHIDE)
      retcode% = grPutPixel%(scp.x%, scp.y%, skcolor%)
      retcode% = grDisplayMouse%(grSHOW)
  
    ' Right press - line to and update CP  
    ELSEIF ((grevent.mask% AND grERPRESS) <> 0) THEN
      retcode% = grMoveTo%(scp.x%, scp.y%)
      retcode% = grDisplayMouse%(grHIDE)
      retcode% = grLineTo%(grevent.curx%, grevent.cury%)
      retcode% = grDisplayMouse%(grSHOW)
      scp.x% = grevent.curx%
      scp.y% = grevent.cury%
    END IF
  
  END IF
  
END SUB ' end of HandleEvent  
  
'******** 
  
'
' Draw windows, buttons and field names.
'
'
'
  
SUB BuildScreen
  
  ' Draw all the windows  
  CALL DrawWindow(mwx%, mwy%, mwx% +MWW-1, maxy%,       "Event Proccessing Example Program", maxcolor%, mwbkc%)
  CALL DrawWindow(EMWX+mwx%, EMWY+mwy%, EMWX+mwx%+EMWW-1, EMWY+mwy%+EMWH-1, "Event Mask", maxcolor%, emwbkc%)
  CALL DrawWindow(QSWX+mwx%, QSWY+mwy%, QSWX+mwx%+QSWW-1, QSWY+mwy%+QSWH-1, "Queue Status", maxcolor%, qswbkc%)
  CALL DrawWindow(SKWX+mwx%, SKWY+mwy%, SKWX+mwx%+SKWW-1, SKWY+mwy%+SKWH-1, "Sketch", maxcolor%, skwbkc%)
  CALL DrawWindow(KBWX+mwx%, KBWY+mwy%, KBWX+mwx%+KBWW-1, KBWY+mwy%+KBWH-1, "Keystrokes", maxcolor%, kbwbkc%)
  
  ' Reset Keyboard and sketch windows  
  CALL ResetKBW
  CALL ResetSKW
  
  ' Draw instructions at bottom of sketch window  
  retcode% = grSetViewPort%(SKWX+mwx%, SKWY+mwy%, maxx%, maxy%)
  retcode% = grMoveTo%(SKWW\2, SKWH-1-VMARGIN-THEIGHT)
  retcode% = grSetBkColor%(skwbkc%)
  retcode% = grOutText%("Use Left and Right buttons to draw")
  retcode% = grDrawLine%(1, SKWH-TITLEHEIGHT-1, SKWW-2, SKWH-TITLEHEIGHT-1)
  
  ' Draw the buttons in the event mask window  
  retcode% = grSetViewPort%(EMWX+mwx%, EMWY+mwy%, maxx%, maxy%)
  CALL DrawButton( BNOEVENTS, BUP)
  CALL DrawButton(BALLEVENTS, BUP)
  CALL DisplayMaskButtons
  
  ' Label the fields in the queue status window  
  retcode% = grSetViewPort%(QSWX+mwx%, QSWY+mwy%, maxx%, maxy%)
  CALL DrawString( 1, 0, "  Last/Current Event", maxcolor%, qswbkc%)
  CALL DrawString( 2, 0, " mask:", maxcolor%, qswbkc%)
  CALL DrawString( 3, 0, "ascii:     scan:", maxcolor%, qswbkc%)
  CALL DrawString( 4, 0, "state:", maxcolor%, qswbkc%)
  CALL DrawString( 5, 0, " curx:     cury:", maxcolor%, qswbkc%)
  CALL DrawString( 6, 0, "stime:", maxcolor%, qswbkc%)
  CALL DrawString( 7, 0, "---------------------", maxcolor%, qswbkc%)
  CALL DrawString( 8, 0, "  Events:    of 25", maxcolor%, qswbkc%)
  CALL DrawString( 9, 0, "Overflow:", maxcolor%, qswbkc%)
  
  ' Draw the remaining buttons, EventProcessing, Reset and Quit  
  CALL DrawButton(BEVENTPROC, BDOWN)
  retcode% = grSetViewPort%(mwx%, mwy%, maxx%, maxy%)
  CALL DrawButton(BRESET, BUP)
  CALL DrawButton( BQUIT, BUP)
  
  CALL UpdateQueueStatus
  
  retcode% = grDisplayMouse%(grSHOW)
  
END SUB ' end of BuildScreen  
  
'******** 
  
'
' Turn off mouse and event queue and return to text mode
'
'
'
  
SUB CleanUp
  
  ' Always turn off mouse driver and events before exiting  
  retcode% = grStopMouse%
  retcode% = grSetEventMask%(grENOEVENTS)
  
END SUB ' end of CleanUp  
  
'******** 
  
'
' Find a valid display type - first try 16 color VGA & EGA, then HERC
'
'
'
  
SUB SelectDisplayType
  
  ' Select display mode  
  
  IF (gxVerifyDisplayType%(gxEGA.10) = gxSUCCESS) THEN
  
    grtype% = gxEGA.10
    retcode% = gxSetDisplay%(grtype%)
  
    maxcolor% = grMaxColor%
  
    mwx% = 0
    mwy% = 0
    mwbkc%   = grBLUE
    emwbkc%  = grRED
    qswbkc%  = grGREEN
    skwbkc%  = grBLACK
    kbwbkc%  = grCYAN
    buttonc% = grGRAY
    bevelc%  = grDARKGRAY
  
  ELSEIF (gxVerifyDisplayType%(gxHERC) = gxSUCCESS) THEN
  
    retcode% = gxSetHerc%(gxHERChalf)
    grtype% = gxHERC
    retcode% = gxSetDisplay%(grtype%)
  
    maxcolor% = grMaxColor%
  
    mwx% = 40
    mwy% = 0
    mwbkc%   = grBLACK
    emwbkc%  = grBLACK
    qswbkc%  = grBLACK
    skwbkc%  = grBLACK
    kbwbkc%  = grBLACK
    buttonc% = maxcolor%
    bevelc%  = grBLACK
  
  ELSEIF (gxVerifyDisplayType%(gxEGA.F) = gxSUCCESS) THEN
  
    grtype% = gxEGA.F
    retcode% = gxSetDisplay%(grtype%)
  
    maxcolor% = grMaxColor%
  
    mwx% = 0
    mwy% = 0
    mwbkc%   = grBLACK
    emwbkc%  = grBLACK
    qswbkc%  = grBLACK
    skwbkc%  = grBLACK
    kbwbkc%  = grBLACK
    buttonc% = maxcolor%
    bevelc%  = grBLACK
  
  ELSE
    grtype% = -1
  
  END IF
  
  maxx% = grMaxX%
  maxy% = grMaxY%
  
END SUB ' end of SelectDisplayType  
  
'******** 
  
'
' Init the mouse, text style, etc.
'
'
'
  
FUNCTION SetUp%
  
  DIM ret%
  DIM mouseavail%
  
  deltaof% = -1                             ' Only updated when nonzero    
  ofcount% = 0                              ' Zero overflow count          
  ecount% = 0                               ' Zero event count             
  ep% = gxTRUE                              ' Start w/event proc. on       
  skcolor% = 1                              ' Skip black                   
  emask% = INITEMASK                        ' Initial value of event mask  
  oldcount% = -1                            ' First count always displayed 
  
  ' Initialize the mouse  
  mouseavail% = grInitMouse%
  ret% = grTrackMouse%(grTRACK)
  
  ' Init text attributes  
  ret% = grSetTextStyle%(grTXT8X8, grOPAQUE)
  ret% = grSetTextJustify%(grTCENTER, grTTOP)
  
  ' Init button positions  
  CALL InitButtons
  
  ' Init click regions  
  CALL InitRects
  
  ' Not finished yet, just getting started  
  done% = gxFALSE
  
  ' Install the event mask  
  ret% = grSetEventMask%(emask%)
  
  SetUp% = mouseavail%
  
END FUNCTION ' end of Setup  
  
'******** 
  
MAIN:
  
  DIM presses%                              ' Left button press count      
  DIM x%, y%                                ' Location of mouse click      
  
  ' Wait for a keypress  
  PRINT"Press a key to run . . ."
  DO
  LOOP WHILE (INKEY$ = "")
  
  ' Find a valid display mode  
  CALL SelectDisplayType
  
  ' Set up the display type and mode  
  retcode% = gxSetMode%(gxGRAPHICS)
  IF (retcode% = gxSUCCESS) THEN
  
    ' Perform misc. initializations including mouse  
    IF (SetUp% = gxSUCCESS) THEN
  
      ' Draw the windows  
      CALL BuildScreen
  
      ' Main loop  
      WHILE (done% <> gxTRUE)
        ' Normally grGetMousePresses isnt needed when using the event queue.
        ' However, in this case since we need to be able to click buttons
        ' even with events turned off, we have to use it.
        '
        retcode% = grGetMousePresses%(grLBUTTON, presses%, x%, y%)
  
        ' Has there been a left button press?  
        IF (presses% <> 0) THEN
  
          ' In the mask button rect?  
          IF (PinR%(x%, y%, maskR) = gxTRUE) THEN
            CALL HandleMaskButtons(x%, y%)
  
          ' In the event processing button rect?  
          ELSEIF (PinR%(x%, y%, epR) = gxTRUE) THEN
            CALL HandleEPButton
  
          ' In the reset button rect?  
          ELSEIF (PinR%(x%, y%, resetR) = gxTRUE) THEN
            CALL HandleResetButton
  
          ' In the quit button rect?  
          ELSEIF (PinR%(x%, y%, quitR) = gxTRUE) THEN
            CALL HandleQuitButton
          ENDIF
        ENDIF
  
        ' Get the number of events in queue and overflows and update screen  
        retcode% = grGetEventStatus%(ecount%, deltaof%)
        ofcount% = ofcount% + deltaof%
        CALL UpdateQueueStatus
  
        ' If event processing on and events in the queue then process them  
        IF ((ep% <> 0) AND (ecount% <> 0)) THEN
          retcode% = grGetEvent%(grevent)
          CALL DisplayEvent(grevent)
          CALL HandleEvent(grevent)
        ENDIF
      WEND
      ' Return to text mode  
      retcode% = gxSetMode%(gxTEXT)
    ELSE
      ' Error during init - cant find mouse  
      retcode% = gxSetMode%(gxTEXT)
      PRINT "grEvents requires a mouse"
    END IF
  
  ELSE
    ' Error setting the mode  
    PRINT "gxSetMode error: ", retcode%
    PRINT "grEvents requires a VGA, EGA or Hercules display"
  END IF
  
  CALL CleanUp
  
END ' end of main  
  
